from abc import ABC, abstractmethod
from typing import List

import pandas as pd

from us_congress._api.models import Congressman


class ICongressDataTransformationService(ABC):
    """
    translates API results into DataFrames
    """

    @abstractmethod
    def congressmembers(self, members: List[Congressman]) -> pd.DataFrame:
        pass



from functools import cache
from logging import Logger
from typing import List, cast

import pandas as pd

from us_congress._api.interface import ICongressApiFetchService
from us_congress._api.models import Congressman
from us_congress._members.interface import ICongressMemberRepository
from us_congress._transformation.interface import ICongressDataTransformationService
from us_congress._utils.log.factory import ILoggerFactory
from us_congress._utils.timer import timer


class CongressMemberRepository(ICongressMemberRepository):
    _api: ICongressApiFetchService
    _transformer: ICongressDataTransformationService
    _logger: Logger

    def __init__(
        self,
        api: ICongressApiFetchService,
        transformer: ICongressDataTransformationService,
        loggerFactor: ILoggerFactory,
    ) -> None:
        self._api = api
        self._transformer = transformer
        self._logger = loggerFactor.getLogger(__name__)

    @timer
    def getRepresentatives(self) -> pd.DataFrame:
        return self.__getRepresentatives()

    @cache
    def __getRepresentatives(self) -> pd.DataFrame:
        self._logger.debug("getting representatives")
        apiRes = cast(List[Congressman], self._api.getRepresentatives())

        return self._transformer.congressmembers(apiRes)

    @timer
    def getSenators(self) -> pd.DataFrame:
        return self.__getSenators()

    @cache
    def __getSenators(self) -> pd.DataFrame:
        self._logger.debug("getting senators")
        apiRes = cast(List[Congressman], self._api.getSenators())

        return self._transformer.congressmembers(apiRes)



from abc import ABC, abstractmethod

import pandas as pd


class ICongressMemberRepository(ABC):
    """
    Gets & stores info for congress members
    """

    @abstractmethod
    def getRepresentatives(self) -> pd.DataFrame:
        ...

    @abstractmethod
    def getSenators(self) -> pd.DataFrame:
        ...



from abc import ABC, abstractmethod
from typing import List

from us_congress._api.models import Representative, Senator


class ICongressApiFetchService(ABC):
    """
    Calls ProPublica API to get basic
    info on congressional leaders
    """

    @abstractmethod
    def getRepresentatives(self) -> List[Representative]:
        ...

    @abstractmethod
    def getSenators(self) -> List[Senator]:
        pass



# pyright: reportUnknownMemberType=false

import os
from typing import cast

import dotenv
import pandas
import punq

from us_congress._api.fetch import CongressApiFetchService
from us_congress._api.interface import ICongressApiFetchService
from us_congress._client import CongressClient
from us_congress._config import CongressConfig
from us_congress._exceptions import NoCongressApiKeyException
from us_congress._members.interface import ICongressMemberRepository
from us_congress._members.service import CongressMemberRepository
from us_congress._transformation.interface import ICongressDataTransformationService
from us_congress._transformation.service import CongressDataTransformationService
from us_congress._utils.log.configureLogger import DEFAULT_LOGFILE, configureLogger
from us_congress._utils.log.factory import ILoggerFactory, LoggerFactory

_transformer = CongressDataTransformationService()
_loggerFactory = LoggerFactory()


class Congress:
    _client: CongressClient
    _config: CongressConfig

    def __init__(self, congressNumber: int, logFile: str = DEFAULT_LOGFILE) -> None:
        dotenvPath = dotenv.find_dotenv()
        dotenv.load_dotenv(dotenvPath)  # type: ignore

        apiKey = os.getenv("PROPUBLICA_CONG_KEY")

        if apiKey is None:
            raise NoCongressApiKeyException(
                "Could not find `PROPUBLICA_CONG_KEY in .env"
            )

        self._config = CongressConfig(congressNumber, apiKey)

        container = punq.Container()

        # singletons
        container.register(ICongressDataTransformationService, instance=_transformer)
        container.register(CongressConfig, instance=self._config)
        container.register(ILoggerFactory, instance=_loggerFactory)

        container.register(ICongressApiFetchService, CongressApiFetchService)
        container.register(ICongressMemberRepository, CongressMemberRepository)

        container.register(CongressClient)

        configureLogger(logFile)

        self._client = cast(CongressClient, container.resolve(CongressClient))

    def getSenators(self) -> pandas.DataFrame:
        return self._client.getSenators().copy(deep=True)

    def getRepresentatives(self) -> pandas.DataFrame:
        return self._client.getRepresentatives().copy(deep=True)

    def __repr__(self) -> str:
        return f"<Congress {self._config.congress}>"



from typing import Any, Dict, List, cast

import requests

from us_congress._api.interface import ICongressApiFetchService
from us_congress._api.models import Representative, Senator
from us_congress._config import CongressConfig

BASE_URL = "https://api.propublica.org/congress/v1"


class CongressApiFetchService(ICongressApiFetchService):
    _config: CongressConfig

    def __init__(self, config: CongressConfig) -> None:
        self._config = config

    def getRepresentatives(self) -> List[Representative]:
        res = self.__fetch("house/members.json")
        members = cast(List[Dict[str, Any]], res["results"][0]["members"])

        return [Representative.fromJson(member) for member in members]

    def getSenators(self) -> List[Senator]:
        res = self.__fetch("senate/members.json")
        members = cast(List[Dict[str, Any]], res["results"][0]["members"])

        return [Senator.fromJson(member) for member in members]

    def __fetch(self, path: str) -> Any:
        headers: Dict[str, str] = {"X-API-Key": self._config.apiKey}

        return requests.get(f"{BASE_URL}/{self._config.congress}/{path}", headers=headers).json()  # type: ignore



import pandas as pd

from us_congress._members.interface import ICongressMemberRepository


class CongressClient:
    """
    client for getting congress info
    """

    _memberRepo: ICongressMemberRepository

    def __init__(self, memberRepo: ICongressMemberRepository) -> None:
        self._memberRepo = memberRepo

    def getSenators(self) -> pd.DataFrame:
        return self._memberRepo.getSenators().copy(deep=True)

    def getRepresentatives(self) -> pd.DataFrame:
        return self._memberRepo.getRepresentatives().copy(deep=True)



from typing import Any, Dict, List

import pandas as pd
import us

from us_congress._api.models import Congressman
from us_congress._transformation.interface import ICongressDataTransformationService


class CongressDataTransformationService(ICongressDataTransformationService):
    def congressmembers(self, members: List[Congressman]) -> pd.DataFrame:
        membersFlat: List[Dict[str, Any]] = []

        for member in members:
            memberDict = {}
            for k, v in member.__dict__.items():
                if hasattr(v, "__dict__"):
                    for k1, v1 in v.__dict__.items():
                        key = f"{k}_{k1}"
                        memberDict.update({key: v1})
                else:
                    memberDict.update({k: v})
                memberDict.update(
                    {"fips": us.states.mapping("abbr", "fips")[member.state]}  # type: ignore
                )
                if "district" in memberDict:
                    memberDict.update({"district": memberDict["district"].zfill(2)})
            membersFlat.append(memberDict)

        sortKeys = ["fips"] + (["district"] if "district" in membersFlat[0] else [])

        return pd.DataFrame(membersFlat).sort_values(by=sortKeys).reset_index(drop=True)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['us_congress',
 'us_congress._api',
 'us_congress._members',
 'us_congress._transformation',
 'us_congress._utils',
 'us_congress._utils.log']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.2.0,<2.0.0',
 'punq>=0.4.1,<0.5.0',
 'python-dotenv>=0.15.0,<0.16.0',
 'requests>=2.25.1,<3.0.0',
 'us>=2.0.2,<3.0.0']

setup_kwargs = {
    'name': 'us-congress',
    'version': '0.1.2a0',
    'description': 'Client to work with US Congress data',
    'long_description': None,
    'author': 'Joel Krim',
    'author_email': 'drawjk705@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from abc import abstractclassmethod
from dataclasses import dataclass
from typing import Any, Dict


@dataclass(frozen=True)
class Congressman:
    @dataclass(frozen=True)
    class _Percents:
        missedVotes: float
        votesWithParty: float
        votesAgainstParty: float

    @dataclass(frozen=True)
    class _Votes:
        total: int
        missed: int
        present: int

    id: str
    title: str
    shortTitle: str
    firstName: str
    middleName: str
    lastName: str
    suffix: str
    party: str
    state: str
    percents: _Percents
    votes: _Votes

    @staticmethod
    def _getPercents(jsonRes: Dict[str, Any]) -> _Percents:
        return Congressman._Percents(
            missedVotes=jsonRes.get("missed_votes_pct") or 0,
            votesWithParty=jsonRes.get("votes_with_party_pct") or 0,
            votesAgainstParty=jsonRes.get("votes_against_party_pct") or 0,
        )

    @staticmethod
    def _getVotes(jsonRes: Dict[str, Any]) -> _Votes:
        return Congressman._Votes(
            total=jsonRes.get("total_votes") or 0,
            missed=jsonRes.get("missed_votes") or 0,
            present=jsonRes.get("total_present") or 0,
        )

    @abstractclassmethod
    def fromJson(cls, jsonRes: Dict[str, Any]) -> Any:
        ...


@dataclass(frozen=True)
class Representative(Congressman):
    district: str

    @classmethod
    def fromJson(cls, jsonRes: Dict[str, Any]):
        percents = Congressman._getPercents(jsonRes)
        votes = Congressman._getVotes(jsonRes)

        return cls(
            id=jsonRes["id"],
            title=jsonRes["title"],
            shortTitle=jsonRes["short_title"],
            firstName=jsonRes["first_name"],
            middleName=jsonRes["middle_name"],
            lastName=jsonRes["last_name"],
            suffix=jsonRes["suffix"],
            party=jsonRes["party"],
            state=jsonRes["state"],
            district=jsonRes["district"],
            percents=percents,
            votes=votes,
        )


@dataclass(frozen=True)
class Senator(Congressman):
    @classmethod
    def fromJson(cls, jsonRes: Dict[str, Any]):
        percents = Congressman._getPercents(jsonRes)
        votes = Congressman._getVotes(jsonRes)

        return cls(
            id=jsonRes["id"],
            title=jsonRes["title"],
            shortTitle=jsonRes["short_title"],
            firstName=jsonRes["first_name"],
            middleName=jsonRes["middle_name"],
            lastName=jsonRes["last_name"],
            suffix=jsonRes["suffix"],
            party=jsonRes["party"],
            state=jsonRes["state"],
            percents=percents,
            votes=votes,
        )
